<?php

/**
 * @file
 * Realm editing UI.
 *
 * We make very little use of the EntityAPI interface for this -
 * preferring instead to use views. That offers more flexibility to change a UI
 * that will, more often than not, be end-user facing.
 */

/**
 * Form callback wrapper: create or edit a realm.
 *
 * @param object $realm
 *   The realm object being edited by this form.
 *
 * @return array
 *   A form.
 *
 * @see realm_edit_form()
 */
function realm_form_wrapper($realm) {

  // Add the breadcrumb for the form's location.
  realm_set_breadcrumb();

  return drupal_get_form('realm_edit_form', $realm);
}

/**
 * Form callback wrapper: delete a realm.
 *
 * @param object $realm
 *   The realm object being edited by this form.
 *
 * @return array|mixed
 *   A form.
 *
 * @see realm_edit_form()
 */
function realm_delete_form_wrapper($realm) {

  return drupal_get_form('realm_delete_form', $realm);
}

/**
 * Form callback: create or edit a realm.
 *
 * @ingroup Forms
 */
function realm_edit_form($form, &$form_state, $realm) {

  // Add the default field elements.
  $form['name'] = array(
    '#type' => 'machine_name',
    '#title' => t('Realm Name'),
    '#default_value' => isset($realm->name) ? $realm->name : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => -5,
    '#machine_name' => array(
      'exists' => 'realm_machine_name_exists',
    ),
  );

  // Add the field related form elements.
  $form_state['realm'] = $realm;
  field_attach_form('realm', $realm, $form, $form_state);

  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('form-actions')),
    '#weight' => 400,
  );

  // We add the form's #submit array to this button along with the actual submit
  // handler to preserve any submit handlers added by a form callback_wrapper.
  $submit = array();

  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save realm'),
    '#submit' => $submit + array('realm_edit_form_submit'),
  );

  if (!empty($realm->name)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete realm'),
      '#suffix' => l(t('Cancel'), 'admin/content/realms'),
      '#submit' => $submit + array('realm_form_submit_delete'),
      '#weight' => 45,
    );
  }

  // We append the validate handler to #validate in case a form callback_wrapper
  // is used to add validate handlers earlier.
  $form['#validate'][] = 'realm_edit_form_validate';

  return $form;
}

/**
 * Form API validate callback for the realm form.
 */
function realm_edit_form_validate(&$form, &$form_state) {

  $realm = $form_state['realm'];

  // Notify field widgets to validate their data.
  field_attach_form_validate('realm', $realm, $form, $form_state);
}

/**
 * Form API submit callback for the realm form.
 *
 * @todo remove hard-coded link
 */
function realm_edit_form_submit(&$form, &$form_state) {

  $realm = entity_ui_controller('realm')->entityFormSubmitBuildEntity($form, $form_state);

  // Add in created and changed times.
  if ($realm->is_new = isset($realm->is_new) ? $realm->is_new : 0) {
    $realm->created = time();
  }

  $realm->changed = time();

  $realm->save();
  $form_state['redirect'] = 'admin/content/realms';
}

/**
 * Form API submit callback for the delete button.
 *
 * @todo Remove hard-coded path
 */
function realm_form_submit_delete(&$form, &$form_state) {

  $form_state['redirect'] = 'admin/content/realms/realm/' . $form_state['realm']->realm_id . '/delete';
}

/**
 * Form callback: confirmation form for deleting a realm.
 *
 * @see confirm_form()
 */
function realm_delete_form($form, &$form_state, $realm) {

  $form_state['realm'] = $realm;

  $form['#submit'][] = 'realm_delete_form_submit';

  $form = confirm_form($form, t('Are you sure you want to delete realm %name?', array('%name' => $realm->name)), 'admin/content/realms/realm', '<p>' . t('This action cannot be undone.') . '</p>', t('Delete'), t('Cancel'), 'confirm');

  return $form;
}

/**
 * Submit callback for realm_delete_form.
 */
function realm_delete_form_submit($form, &$form_state) {

  $realm = $form_state['realm'];

  realm_delete($realm);

  drupal_set_message(t('The realm %name has been deleted.', array('%name' => $realm->name)));
  watchdog('realm', 'Deleted realm %name.', array('%name' => $realm->name));

  $form_state['redirect'] = 'admin/content/realms';
}

/**
 * Page to add Realm Entities.
 *
 * @todo Pass this through a proper theme function
 */
function realm_add_page() {

  $controller = entity_ui_controller('realm');

  return $controller->addPage();
}

/**
 * Displays the list of available realm types for realm creation.
 *
 * @ingroup themeable
 */
function theme_realm_add_list($variables) {

  $content = $variables['content'];
  $output = '';
  if ($content) {
    $output = '<dl class="realm-type-list">';
    foreach ($content as $item) {
      $output .= '<dt>' . l($item['title'], $item['href']) . '</dt>';
      $output .= '<dd>' . filter_xss_admin($item['description']) . '</dd>';
    }
    $output .= '</dl>';
  }
  else {
    if (user_access('administer realm types')) {
      $output = '<p>' . t('Realm Entities cannot be added because you have not created any realm types yet. Go to the <a href="@create-realm-type">realm type creation page</a> to add a new realm type.', array('@create-realm-type' => url('admin/structure/realm_types/add'))) . '</p>';
    }
    else {
      $output = '<p>' . t('No realm types have been created yet for you to use.') . '</p>';
    }
  }

  return $output;
}

/**
 * Sets the breadcrumb for administrative realm pages.
 */
function realm_set_breadcrumb() {

  $breadcrumb = array(
    l(t('Home'), '<front>'),
    l(t('Administration'), 'admin'),
    l(t('Content'), 'admin/content'),
    l(t('Realms'), 'admin/content/realms'),
  );

  drupal_set_breadcrumb($breadcrumb);
}
