<?php

/**
 * @file
 * Callbacks and other functions for realm.
 */

/**
 * Determines whether the given user has access to a realm.
 *
 * @param string $op
 *   The operation being performed. One of 'view', 'update', 'create', 'delete'
 *   or just 'edit' (being the same as 'create' or 'update').
 * @param object $realm
 *   (Optional) A realm or a realm type to check access for. If nothing is
 *   given, access for all realms is determined.
 * @param object $account
 *   (Optional) The user to check for. Leave it to NULL to check for the global
 *   user.
 *
 * @return bool
 *   Whether access is allowed or not.
 */
function realm_access($op, $realm = NULL, $account = NULL) {

  if (user_access('administer realms', $account)) {
    return TRUE;
  }
  if (isset($realm) && is_object($realm) && $type_name = $realm->type) {
    $op = ($op == 'view') ? 'view' : 'edit';
    if (user_access("$op any $type_name realm", $account)) {
      return TRUE;
    }
  }

  return FALSE;
}

/**
 * Access callback for the entity API.
 */
function realm_type_access($op, $type = NULL, $account = NULL) {

  return user_access('administer realm types', $account);
}

/**
 * Gets an array of all realm types, keyed by the type name.
 *
 * @param string $type_name
 *   (Optional) The type with the given name is returned.
 *
 * @return \Drupal\realm\Entity\RealmType|array
 *   Depending whether $type is set, an array of realm types or a single one.
 */
function realm_get_types($type_name = NULL) {

  // entity_load() will get the Entity controller for our realm entity and call
  // the load function of that object - we are loading entities by name here.
  $types = entity_load_multiple_by_name('realm_type', isset($type_name) ? array($type_name) : FALSE);

  return isset($type_name) ? reset($types) : $types;
}

/**
 * Menu argument loader; Load a realm type by string.
 *
 * @param string $type
 *   The machine-readable name of a realm type to load.
 *
 * @return \Drupal\realm\Entity\RealmType|array
 *   A realm type array or FALSE if $type does not exist.
 */
function realm_type_load($type) {

  return realm_get_types($type);
}

/**
 * Fetch a realm object.
 *
 * Make sure that the wildcard you choose in the realm entity definition fits
 * the function name here.
 *
 * @param string|int $realm_id
 *   Integer specifying the realm id.
 * @param bool $reset
 *   (Optional) A boolean indicating that the internal cache should be reset.
 *
 * @return \Drupal\realm\Entity\Realm|bool
 *   A fully-loaded $realm object or FALSE if it cannot be loaded.
 *
 * @see realm_load_multiple()
 */
function realm_load($realm_id, $reset = FALSE) {

  $realms = realm_load_multiple(array($realm_id), array(), $reset);

  return reset($realms);
}

/**
 * Load multiple realms based on certain conditions.
 *
 * @param array|bool $realm_ids
 *   (Optional) An array of realm IDs, for FALSE to load all of them.
 * @param array $conditions
 *   (Optional) An array of conditions to match against the {realm} table.
 * @param bool $reset
 *   (Optional) A boolean indicating that the internal cache should be reset.
 *
 * @return array
 *   An array of realm objects, indexed by realm_id.
 *
 * @see entity_load()
 * @see realm_load()
 */
function realm_load_multiple($realm_ids = FALSE, $conditions = array(), $reset = FALSE) {

  return entity_load('realm', $realm_ids, $conditions, $reset);
}

/**
 * Deletes a realm.
 */
function realm_delete(\Drupal\realm\Entity\Realm $realm) {

  $realm->delete();
}

/**
 * Delete multiple realms.
 *
 * @param array $realm_ids
 *   An array of realm IDs.
 */
function realm_delete_multiple(array $realm_ids) {

  entity_get_controller('realm')->delete($realm_ids);
}

/**
 * Create a realm object.
 *
 * @param array $values
 *   (Optional) The values to create.
 */
function realm_create($values = array()) {

  return entity_get_controller('realm')->create($values);
}

/**
 * Saves a realm to the database.
 *
 * @param \Drupal\realm\Entity\Realm $realm
 *   The realm object.
 */
function realm_save(\Drupal\realm\Entity\Realm $realm) {

  return $realm->save();
}

/**
 * Saves a realm type to the db.
 *
 * @param \Drupal\realm\Entity\RealmType $type
 *   A realm type.
 */
function realm_type_save(\Drupal\realm\Entity\RealmType $type) {

  $type->save();
}

/**
 * Deletes a realm type from the db.
 *
 * @param \Drupal\realm\Entity\RealmType $type
 *   A realm type.
 */
function realm_type_delete(\Drupal\realm\Entity\RealmType $type) {

  $type->delete();
}

/**
 * URI callback for realms.
 *
 * @param \Drupal\realm\Entity\Realm $realm
 *   A realm.
 *
 * @return array
 *   An array containing the path.
 */
function realm_uri(\Drupal\realm\Entity\Realm $realm) {

  return array(
    'path' => 'realm/' . $realm->realm_id,
  );
}

/**
 * Menu title callback for showing individual entities.
 *
 * @param \Drupal\realm\Entity\Realm $realm
 *   A realm.
 *
 * @return string
 *   The menu name.
 */
function realm_page_title(Drupal\realm\Entity\Realm $realm) {

  return $realm->name;
}

/**
 * Sets up content to show an individual realm.
 *
 * @todo - get rid of drupal_set_title();
 */
function realm_page_view($realm, $view_mode = 'full') {

  $controller = entity_get_controller('realm');
  $content = $controller->view(array($realm->realm_id => $realm), $view_mode);
  drupal_set_title($realm->label());

  return $content;
}
